package com.geronimolive.gcal.dao;

import com.geronimolive.gcal.model.Event;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.orm.ObjectRetrievalFailureException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * An event DAO that stores events in memory (as a map)
 * 
 * @author Tom McQueeney
 */
public class EventDAOMemory implements EventDAO {

    private static final Log LOG = LogFactory.getLog(EventDAOMemory.class);

    private static final Map events = Collections.synchronizedMap(new LinkedHashMap(97));
    private static long eventId = 0;
    

    public List getEvents() {
        List l = new ArrayList(events.values());
        LOG.debug("Returning an event list of size: " + l.size());
        
        return Collections.unmodifiableList(l);
    }

    public Event getEvent(Long eventId) {
        Event event = (Event) events.get(eventId);
        if (event == null) {
            LOG.info(
                "Failed to find eventId '" + eventId + 
                "'. Throwing retrieval exception"
            );
            throw new ObjectRetrievalFailureException(Event.class, eventId);
        }
        return event;
    }

    public void saveEvent(Event event) {
        if (event.getId() == null) {
            synchronized (events) {
                event.setId(new Long(++eventId)); // Create primary key for event.
            }
        } else if (!events.containsKey(event.getId())) {
            throw new DataIntegrityViolationException(
                "Attempt to update event not already in database, key=" + 
                event.getId()
            );
        }
        LOG.debug("Saving event Id '" + event.getId() + "' description: " + event.getDescription());
        events.put(event.getId(), event);
    }

    public void removeEvent(Long eventId) {
        LOG.info("Removing event " + eventId + " from events of size: " + events.size());
        events.remove(eventId);
        LOG.info("Events size is now: " + events.size());
    }

}
