package com.geronimolive.gcal.web;

import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.converters.LongConverter;
import org.apache.commons.beanutils.locale.converters.DateLocaleConverter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.DynaActionForm;
import org.apache.struts.actions.DispatchAction;
import org.apache.struts.util.MessageResources;

import com.geronimolive.gcal.model.Event;
import com.geronimolive.gcal.service.EventManager;

/**
 * Struts dispatch action to handle calendar events.
 *
 * @author Tom McQueeney
 */
public class EventAction extends DispatchAction {

    static {
        ConvertUtils.register(new LongConverter(null), Long.class);
    }
    
    private static final Log LOG = LogFactory.getLog(EventAction.class);

    private EventManager mgr = null;

    /**
     * Spring sets an event manager when instantiating this class.
     * @param eventManager
     *     an event manager that will be used to manage events.
     */
    public void setEventManager(EventManager eventManager) {
        this.mgr = eventManager;
    }

    /**
     * Called by Struts to save a new or updated event into the database.
     */
    public ActionForward save(
        ActionMapping mapping, ActionForm form,
        HttpServletRequest request, HttpServletResponse response
    ) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("entering 'save' method...");
        }

        // Return to event list if the user canceled the save.
        if (isCancelled(request)) {
            return list(mapping, form, request, response);
        }

        ActionMessages errors = form.validate(mapping, request);
        if (!errors.isEmpty()) {
            saveErrors(request, errors);
            return mapping.findForward("edit");
        }

        DynaActionForm eventForm = (DynaActionForm) form;
        Event event = (Event) eventForm.get("event");
        mgr.saveEvent(event);

        ActionMessages messages = new ActionMessages();
        messages.add(
            ActionMessages.GLOBAL_MESSAGE, 
            new ActionMessage("event.saved", event.getDescription())
        );
        saveMessages(request.getSession(), messages);

        return mapping.findForward("events");
    }

    /**
     * Called by Struts when user asks to edit or add an event.
     */
    public ActionForward edit(
        ActionMapping mapping, ActionForm form,
        HttpServletRequest request, HttpServletResponse response
    ) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("Entering 'edit' method");
        }

        DynaActionForm eventForm = (DynaActionForm) form;
        String eventId = request.getParameter("id");

        // A null eventId indicates this is a new event to add.
        if (eventId != null) {
            Event event = mgr.getEvent(eventId);

            if (event == null) {
                ActionMessages errors = new ActionMessages();
                errors.add(
                    ActionMessages.GLOBAL_MESSAGE, 
                    new ActionMessage("event.missing")
                );
                saveErrors(request, errors);

                return mapping.findForward("list");
            }

            eventForm.set("event", event);
        }

        MessageResources resources = getResources(request);

        // Register date editor to handle date conversions.
        DateLocaleConverter dateConverter = new DateLocaleConverter(
            null, request.getLocale(), resources.getMessage("date.format")
        );
        dateConverter.setLenient(true);
        ConvertUtils.register(dateConverter, Date.class);

        return mapping.findForward("edit");
    }

    /**
     * Called by Struts to remove an event from the database.
     */
    public ActionForward delete(
        ActionMapping mapping, ActionForm form,
        HttpServletRequest request, HttpServletResponse response
    ) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("Entering 'delete' method");
        }

        DynaActionForm eventForm = (DynaActionForm) form;
        Event event = (Event) eventForm.get("event");

        mgr.removeEvent(request.getParameter("event.id"));

        ActionMessages messages = new ActionMessages();
        messages.add(
            ActionMessages.GLOBAL_MESSAGE, 
            new ActionMessage("event.deleted", event.getDescription())
        );

        saveMessages(request.getSession(), messages);

        return mapping.findForward("events");
    }

    /**
     * Called by Struts to display a list of all events in the calendar.
     */
    public ActionForward list(
        ActionMapping mapping, ActionForm form,
        HttpServletRequest request, HttpServletResponse response
    ) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("Entering 'list' method");
        }

        request.setAttribute("events", mgr.getEvents());

        return mapping.findForward("list");
    }

    /**
     * List events when no parameter is used.
     */
    public ActionForward unspecified(
        ActionMapping mapping, ActionForm form,
        HttpServletRequest request, HttpServletResponse response
    ) {
        return list(mapping, form, request, response);
    }

}
