package com.geronimolive.gcal.web;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

/**
 * A servlet filter to remove Struts success messages from the session scope and
 * put them in the request scope. The success messages were saved in the session
 * scope to avoid losing them in a redirect, but they don't need to stay there
 * so try to keep the session clean. Thank you to Matt Raible for this technique.
 * 
 * @web.filter
 *   name="messageFilter"
 *   description="Moves Struts success messages from session scope to request scope"
 * @web.filter-mapping
 *     url-pattern="/*"
 */
public class MessageFilter implements Filter {
    public void doFilter(
        ServletRequest req, ServletResponse res, FilterChain chain
    ) throws IOException, ServletException 
    {
        HttpServletRequest request = (HttpServletRequest) req;
        Object message = request.getSession().getAttribute("message");

        if (message != null) {
            request.setAttribute("message", message);
            request.getSession().removeAttribute("message");
        }

        chain.doFilter(req, res);
    }

    public void init(FilterConfig filterConfig) {
        // empty
    }

    public void destroy() {
        // empty
    }
}
