package com.geronimolive.gcal.dao;

import org.springframework.dao.DataAccessException;

import java.util.Date;

import com.geronimolive.gcal.dao.EventDAO;
import com.geronimolive.gcal.model.Event;

public class EventDAOTest extends BaseDAOTestCase {
    private EventDAO dao = null;

    protected void setUp() throws Exception {
        super.setUp();
        dao = (EventDAO) ctx.getBean("eventDAO");
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        dao = null;
    }

    private Event createDummyEvent() {
        Event event = new Event();
        event.setDescription("Test event");
        event.setStartDateTime(new Date());
        event.setEndDateTime(new Date());
        return event;
    }

    public void testGetEvents() {
        dao.saveEvent(createDummyEvent());
        assertTrue(dao.getEvents().size() > 0);
    }

    public void testSaveEvent() {
        Event event = createDummyEvent();
        dao.saveEvent(event);
        assertNotNull("primary key failed to get assigned", event.getId());
        assertNotNull(event.getDescription());
    }

    public void testSaveAndEditEvent() {
        Event event = createDummyEvent();
        dao.saveEvent(event);
        final int size1 = dao.getEvents().size();
        event = dao.getEvent(event.getId());
        assertEquals(
            "Retrieved event not what was stored", 
            "Test event", event.getDescription()
        );
        event.setDescription("New event description");
        dao.saveEvent(event);
        final int size2 = dao.getEvents().size();
        assertEquals("Saving same event added a new event", size1, size2);
        event = dao.getEvent(event.getId());
        assertEquals(
            "Edited event not saved as expected", 
            "New event description", event.getDescription()
        );
    }

    public void testAddAndRemoveEvent() {
        Event event = createDummyEvent();
        dao.saveEvent(event);

        assertNotNull(event.getId());
        assertEquals(event.getDescription(), "Test event");

        dao.removeEvent(event.getId());

        try {
            event = dao.getEvent(event.getId());
            fail("Event found in db after removal");
        } catch (DataAccessException dae) {
            // Expected
            assertNotNull(dae);
        }
    }

    public void testSavingIllegalEvent() {
        Event event = createDummyEvent();
        event.setId(new Long(dao.getEvents().size() + 10)); // Shouldn't create own key
        try {
            dao.saveEvent(event);
            fail("Should have thrown exception saving event with own key");
        } catch (DataAccessException dae) {
            // expected
        }
    }
    
}
