package com.geronimolive.gcal.service;

import org.jmock.Mock;
import org.jmock.MockObjectTestCase;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.ObjectRetrievalFailureException;

import java.util.Date;

import com.geronimolive.gcal.dao.EventDAO;
import com.geronimolive.gcal.model.Event;
import com.geronimolive.gcal.service.EventManager;

public class EventManagerTest extends MockObjectTestCase {
    private EventManager mgr = new DefaultEventManager();
    private Mock mockDAO = null;

    protected void setUp() throws Exception {
        mockDAO = new Mock(EventDAO.class);
        mgr.setEventDAO((EventDAO) mockDAO.proxy());
    }

    private Event createDummyEvent() {
        Event event = new Event();
        event.setDescription("Test event");
        event.setStartDateTime(new Date());
        event.setEndDateTime(new Date());
        return event;
    }

    public void testAddAndRemoveEvent() {
        Event event = createDummyEvent();

        // set expected behavior on dao
        mockDAO.expects(once()).method("saveEvent").with(same(event));

        mgr.saveEvent(event);

        // verify expectations
        mockDAO.verify();

        assertEquals(event.getDescription(), "Test event");

        mockDAO.expects(once()).method("removeEvent").with(eq(new Long(1)));
        mgr.removeEvent("1");

        // verify expectations
        mockDAO.verify();

        try {
            // Set expectations
            Throwable ex = 
                new ObjectRetrievalFailureException(Event.class, "1");
            mockDAO.expects(once()).method("getEvent").with(eq(new Long(1))).will(
                throwException(ex)
            );

            event = mgr.getEvent("1");

            // Verify it throws exception
            mockDAO.verify();

            fail("Test event found in database after removal");
        } catch (DataAccessException dae) {
            // Expected
            assertNotNull(dae);
        }
    }
}
