package com.geronimolive.gcal.web;

import java.util.Date;
import java.util.List;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.geronimolive.gcal.model.Event;
import com.geronimolive.gcal.service.EventManager;

import servletunit.struts.MockStrutsTestCase;

public class EventActionTest extends MockStrutsTestCase {
    private Long eventId = null;


    public EventActionTest(String testName) {
        super(testName);
    }

    /**
     * Setup by configuring Spring, getting the eventManager bean, then
     * creating and saving an event for use by the tests.
     */
    public void setUp() throws Exception {
        super.setUp();
        String[] paths = {"/WEB-INF/applicationContext*.xml"};
        ApplicationContext ctx = new ClassPathXmlApplicationContext(paths);
        EventManager mgr = (EventManager) ctx.getBean("eventManager");
        Event event = createDummyEvent();
        mgr.saveEvent(event);
        this.eventId = event.getId();
    }
    
    private static Event createDummyEvent() {
        Event event = new Event();
        event.setDescription("Test event");
        event.setStartDateTime(new Date());
        event.setEndDateTime(new Date());
        event.setLink("http://test.com");
        return event;
    }

    /**
     * Edit a newly added event
     */
    public void testEdit() {
        setRequestPathInfo("/event");
        addRequestParameter("method", "edit");
        addRequestParameter("id", eventId.toString());
        actionPerform();
        verifyForward("edit");
        verifyNoActionErrors();
    }

    /**
     * Delete a new event
     */
    public void testDelete() {
        setRequestPathInfo("/event");
        addRequestParameter("method", "delete");
        addRequestParameter("event.id", eventId.toString());
        actionPerform();
        verifyForward("events");
        verifyNoActionErrors();
    }

    /**
     * List events to ensure it returned info. 
     */
    public void testList() {        
        setRequestPathInfo("/event");
        addRequestParameter("method", "list");
        actionPerform();
        verifyForward("list");
        verifyNoActionErrors();

        List events = (List) getRequest().getAttribute("events");
        assertNotNull(events);
        assertTrue(events.size() > 0);
    }
}
