package com.geronimolive.gcal.web;

import java.util.List;

import net.sourceforge.jwebunit.WebTestCase;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.geronimolive.gcal.model.Event;
import com.geronimolive.gcal.service.EventManager;

public class UserWebTest extends WebTestCase {

    public UserWebTest(String name) {
        super(name);
        getTestContext().setBaseUrl("http://localhost:8080/gcal");
        getTestContext().setResourceBundleName("messages");
    }

    public void testWelcomePage() {
        beginAt("/");
        assertTitleEqualsKey("index.title");
    }

    public void testAddUser() {
        beginAt("/user.html?method=edit");
        assertTitleEqualsKey("userForm.title");
        setFormElement("user.firstName", "Spring");
        setFormElement("user.lastName", "User");
        submit("save");
        assertTitleEqualsKey("userList.title");
    }

    public void testListUsers() {
        beginAt("/users.html");

        // check that table is present
        assertTablePresent("userList");

        //check that a set of strings are present somewhere in table
        assertTextInTable("userList",
                new String[]{"Spring", "User"});
    }

    public void testEditUser() {
        beginAt("/user.html?method=edit&id=" + getInsertedUserId());
        assertFormElementEquals("user.firstName", "Spring");
        submit("save");
        assertTitleEqualsKey("userList.title");
    }

    public void testDeleteUser() {
        beginAt("/user.html?method=edit&id=" + getInsertedUserId());
        assertTitleEqualsKey("userForm.title");
        submit("delete");
        assertTitleEqualsKey("userList.title");
    }

    /**
     * Convenience method to get the id of the inserted user
     * Assumes last inserted user is "Spring User"
     */
    private String getInsertedUserId() {
        String[] paths = {"/WEB-INF/applicationContext*.xml"};
        ApplicationContext ctx = new ClassPathXmlApplicationContext(paths);
        List users = ((EventManager) ctx.getBean("userManager")).getEvents();
        // assumed that user inserted in testAddUser() is last user
        return "" + ((Event) users.get(users.size() - 1)).getId();
    }
}
